SystemOrganization addCategory: #NPB!
SystemOrganization addCategory: #'NPB-IS'!
SystemOrganization addCategory: #'NPB-Tests'!

Object subclass: #NPBISBase
	instanceVariableNames: 'testIndexArray testRankArray masterHist keyArray partialVerifyVals sizeClass totalKeys maxKey numBuckets numKeys sizeOfBuffers timerOn timer totalKeysLog2 maxKeyLog2 numBucketsLog2 numThreads rankThreads master benchmarkClass mtxNotify mtxCritical'
	classVariableNames: 'PassedVerification'
	poolDictionaries: ''
	category: 'NPB-IS'!

!NPBISBase commentStamp: 'StefanMarr 5/14/2011 01:03' prior: 0!
NPBISBase definines the commonalities for the general benchmark and its actual worker-threads.

See NPBSuite>>benchIS for the actual benchmark implemented based on SMark.

Instance Variables:
	testIndexArray	<Object>
	testRankArray	<Object>
	masterHist	<(SequenceableCollection of: Object)>
	keyArray	<(Array of: (CharacterBlock | Magnitude | Point | String | UUID))>
	partialVerifyVals	<Array>
	sizeClass	<ProtoObject | PseudoContext>
	totalKeys	<Collection | Duration | Number | Point>
	maxKey	<Collection | Duration | Number | Point>
	numBuckets	<ProtoObject | PseudoContext>
	numKeys	<Collection | Color | DateAndTime | Duration | Number | OBMetaNode | Point | Timespan | TraitComposition | TraitDescription | TraitTransformation>
	sizeOfBuffers	<ProtoObject | PseudoContext>
	timerOn	<ProtoObject | PseudoContext>
	timer	<ProtoObject | PseudoContext>
	totalKeysLog2	<ProtoObject | PseudoContext>
	maxKeyLog2	<ProtoObject | PseudoContext>
	numBucketsLog2	<ProtoObject | PseudoContext>
	numThreads	<ProtoObject | PseudoContext>
	rankThreads	<(SequenceableCollection of: RankThread)>
	master	<NPBISBase>
	benchmarkClass	<ProtoObject>
	mtxNotify	<Semaphore>
	mtxCritical	<Semaphore>!

NPBISBase subclass: #NPBIS
	instanceVariableNames: 'bid results serial rng runner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NPB-IS'!

!NPBIS commentStamp: 'StefanMarr 5/14/2011 01:04' prior: 0!
NPBIS definines the general benchmark Integer Sort benchmark of the NAS Parallel Benchmarks.

See NPBSuite>>benchIS for the actual benchmark implemented based on SMark.

Instance Variables:
	bid	<ProtoObject | PseudoContext>
	results	<ProtoObject | PseudoContext>
	serial	<Boolean>
	rng	<NPBRandom>
	runner	<SMarkRunner>!

!NPBIS class methodsFor: 'constants' stamp: 'StefanMarr 9/21/2009 18:15'!
amult
	^1220703125.0! !

!NPBIS class methodsFor: 'constants' stamp: 'StefanMarr 5/14/2011 10:39'!
defaultProblemSize
	^ #S! !

!NPBIS class methodsFor: 'instance creation' stamp: 'StefanMarr 9/23/2009 19:12'!
newOfSizeClass: aSymbol with: nThreads serial: aBool
	| o |
	o := super new.
	
	o initializeWithSizeClass: aSymbol andNumberOfThreads: nThreads.
	
	o serial: aBool.
	o rng: NPBRandom new.
	^o
	
	
! !

!NPBIS methodsFor: 'as yet unclassified' stamp: 'sm 6/1/2011 15:57'!
do: aStep
	self critical: [
		rankThreads do: [ :thread |
			thread startStep: aStep.].
	
		rankThreads do: [ :ignored | 
			self wait.
		]
	].! !

!NPBIS methodsFor: 'as yet unclassified' stamp: 'sm 6/1/2011 14:22'!
fullVerify
	" To save copy and memory sorting can be done directly: "
	| key idx count |
	key := 1. idx := 1.
	
	1 to: numKeys do: [ :i | 
		[ (idx == (masterHist at: key))
		   and: [ ((key >= maxKey)	or: [ idx >= numKeys ]) not ] ] whileTrue: [
			key := key + 1 
		].
		keyArray at: idx put: key.
		idx := idx + 1.
	].

	" Confirm keys correctly sorted: count incorrectly sorted keys, if any "
	count := 0.
	
	2 to: numKeys do: [ :i | 
		((keyArray at: (i - 1)) > (keyArray at: i)) ifTrue: [ count := count + 1]].

	(count ~= 0)
		ifTrue: [ Error signal: 'Full_verify: number of keys out of sort: '. count asString ]
		ifFalse: [ PassedVerification := PassedVerification + 1].
    
    ^ PassedVerification! !

!NPBIS methodsFor: 'as yet unclassified' stamp: 'StefanMarr 10/16/2009 00:23'!
initKeys: aDouble
	| x k |
	k := maxKey / 4.
	
	1 to: numKeys do: [ :i |
		(i == 32769) ifTrue: [ Transcript show: 'Seed at: ', (rng seed asString);cr.].
		x := rng randlc: aDouble. 
		x := x + (rng randlc: aDouble).
		x := x + (rng randlc: aDouble).
		x := x + (rng randlc: aDouble).
		
		keyArray at: i put: (x * k) asInteger.
		].! !

!NPBIS methodsFor: 'accessors' stamp: 'StefanMarr 9/23/2009 19:14'!
keyArray
	^keyArray ! !

!NPBIS methodsFor: 'license' stamp: 'StefanMarr 4/15/2011 17:04'!
license
	^'This code is based on the Java version of NPB3.

Permission to use, copy, distribute and modify this software for any purpose
with or without fee is hereby granted. We request, however, that all derived
work reference the NAS Parallel Benchmarks 3.0. This software is provided "as
is" without express or implied warranty.

Information on NPB 3.0, including the Technical Report NAS-02-008
"Implementation of the NAS Parallel Benchmarks in Java", original
specifications, source code, results and information on how to submit new
results, is available at:

   http://www.nas.nasa.gov/Software/NPB/'! !

!NPBIS methodsFor: 'accessors' stamp: 'StefanMarr 9/23/2009 19:15'!
masterHist
	^masterHist ! !

!NPBIS methodsFor: 'as yet unclassified' stamp: 'sm 6/1/2011 14:23'!
partialVerify: iteration
	| k offset |

	1 to: NPBISBase testArraySize do: [ :i |
		k := partialVerifyVals at: i.
		offset := iteration.
		( (0 <= k) and: [k <= (numKeys - 1)]) ifTrue: [
			(benchmarkClass == #S) ifTrue: [
					((i - 1) <= 2) ifTrue: [ offset := iteration. ]
						    ifFalse: [ offset := 0 - iteration.].
				].
			
			((masterHist at: k) ~= ((testRankArray at: i) + offset))
				ifTrue: [
					Error signal: 'Failed partial verification: iteration ', iteration asString, ', test key ', i asString ]
				ifFalse: [
					NPBISBase passedVerification: (NPBISBase passedVerification + 1). ]
			]
		]! !

!NPBIS methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 17:18'!
rank: iteration
	keyArray at: (iteration + 1) put: iteration.
	keyArray at: (1 + iteration + (NPBISBase maxIterations)) put: (maxKey - iteration).
	
	1 to: NPBISBase testArraySize do: [
		:i | partialVerifyVals at: i put: (keyArray at: (1 + (testIndexArray at: i)) ) ].
	
	" Clear the work array "
	1 to: maxKey do: [ :i | masterHist at: i put: 0 ].
	
	" In the section, the keys themselves are used as their
	  own indexes to determine how many of each there are: their
	  individual population "
	1 to: numKeys do: [ :i | masterHist at: (1 + (keyArray at: i)) incrementBy: 1 ].
	" Now they have individual key population "
	
	" Density to Distribution conversion "
	1 to: (maxKey - 1) do: [ :i | masterHist at: (i + 1) incrementBy: (masterHist at: i). ].
	
	self partialVerify: iteration.! !

!NPBIS methodsFor: 'as yet unclassified' stamp: 'StefanMarr 9/21/2009 23:31'!
resetMasterHist
	1 to: maxKey do: [ :i | masterHist at: i put: 0 ].! !

!NPBIS methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 23:00'!
rng: val
	rng := val! !

!NPBIS methodsFor: 'as yet unclassified' stamp: 'sm 6/1/2011 14:24'!
run
	| mainTimer tPhase1 tPhase2 tKeyInit tVerify executionVerified |

	ScriptConsole println: ' Size: ', totalKeys asString, ' Iterations: ', NPBISBase maxIterations asString.
	
	mainTimer := runner createTimer: 'precise-total'.
	tPhase1 := runner createTimer: 'phase1'.
	tPhase2 := runner createTimer: 'phase2'.
	tKeyInit := runner createTimer: 'keyInit'.
	tVerify := runner createTimer: 'verifyCompletely'.
	
	NPBISBase passedVerification: 0.
	
	serial
		ifTrue: [
			tKeyInit start.
			self initKeys: NPBIS amult.
			tKeyInit stop. 
			
			self rank: 1. ]
		ifFalse: [
			self setupThreads.
			
			tKeyInit start.
			self do: #initKeys.
			tKeyInit stop.
			
			RankThread iteration: 1.
			self do: #step1.
			
			self resetMasterHist.
			
			self do: #step2.
			self partialVerify: 1].
		
	NPBISBase passedVerification: 0.
		
	mainTimer start.
		
	1 to: NPBIS maxIterations do: [:i | 
		serial
			ifTrue: [ self rank: i. ]
			ifFalse: [ 
				RankThread iteration: i.
				tPhase1 start.
				self do: #step1.
				tPhase1 stop.
				
				self resetMasterHist.
				
				tPhase2 start.
				self do: #step2.
				tPhase2 stop.
				
				self partialVerify: i]
		].
	
	mainTimer stop.
	
	tVerify start.
	self fullVerify.
	tVerify stop.
	
	executionVerified := ((NPBISBase passedVerification) == ((5 * NPBIS maxIterations) + 1)).
	executionVerified
		ifFalse: [ Error signal: 'Benchmark verification failed'].
		
	serial ifFalse: [ self do: #quit. ].
	^ executionVerified! !

!NPBIS methodsFor: 'accessors' stamp: 'StefanMarr 5/14/2011 01:00'!
runner: aRunner
	runner := aRunner
	! !

!NPBIS methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 22:59'!
serial: val
	serial := val! !

!NPBISBase class methodsFor: 'constants' stamp: 'SM 9/16/2009 22:28'!
S
	^ Dictionary newFrom: {
		#testIndexArray -> { 48427. 17148. 23627. 62548. 4431 }.
		#testRankArray -> { 0. 18. 346. 64917. 65463 }.
		#totalKeysLog2 -> 16.
		#maxKeyLog2 -> 11.
		#numBucketsLog2 -> 9
	 }! !

!NPBISBase class methodsFor: 'constants' stamp: 'SM 9/16/2009 22:16'!
maxIterations
	^ 10! !

!NPBISBase class methodsFor: 'instance creation' stamp: 'StefanMarr 9/21/2009 18:28'!
newOfSizeClass: aSymbol with: numberOfThreads 
	| o |
	o := super new.
	o initializeWithSizeClass: aSymbol andNumberOfThreads: numberOfThreads.
	^o! !

!NPBISBase class methodsFor: 'constants' stamp: 'StefanMarr 1/16/2011 21:40'!
passedVerification
	^PassedVerification! !

!NPBISBase class methodsFor: 'constants' stamp: 'StefanMarr 1/16/2011 21:41'!
passedVerification: val
	PassedVerification := val! !

!NPBISBase class methodsFor: 'constants' stamp: 'SM 9/16/2009 22:17'!
testArraySize
	^ 5! !

!NPBISBase methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/7/2010 12:31'!
checksum: intArray name: aString stop: aBool
	| check |
	
	check := intArray sum.
	1 halt.
	"RVMOperations print: name, ' checksum is ', check asString."
	
	aBool ifTrue: [
		1 halt.
		self StopSystemHere]! !

!NPBISBase methodsFor: 'synchronization' stamp: 'StefanMarr 10/5/2009 18:13'!
critical: aBlock
	mtxCritical critical: aBlock! !

!NPBISBase methodsFor: 'initialization' stamp: 'StefanMarr 10/5/2009 18:14'!
initialize
	mtxCritical := Semaphore forMutualExclusion.
	mtxNotify := Semaphore new.! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 4/15/2011 17:19'!
initializeValues
	"initialize the object after construction"
	totalKeys := 1 << self totalKeysLog2.
	maxKey := 1 << maxKeyLog2.
	numKeys := totalKeys.
	sizeOfBuffers := totalKeys.
	keyArray := Array new: sizeOfBuffers.
	masterHist := Array new: maxKey. 
	partialVerifyVals := Array new: NPBISBase testArraySize.
	masterHist doWithIndex: 
		[ :val :i | 
		masterHist 
			at: i
			put: 0 ]! !

!NPBISBase methodsFor: 'initialization' stamp: 'StefanMarr 5/14/2011 10:44'!
initializeWithSizeClass: aSymbol andNumberOfThreads: numberOfThreads 
	| consts |
	benchmarkClass := aSymbol asSymbol.
		
	consts := NPBISBase perform: benchmarkClass.
	consts keysAndValuesDo: 
		[ :key :val | 
		self
			perform: (key , ':') asSymbol
			with: val ].
	numThreads := numberOfThreads.
	
	self initializeValues.
	^ self! !

!NPBISBase methodsFor: 'license' stamp: 'StefanMarr 4/15/2011 17:04'!
license
	^'This code is based on the Java version of NPB3.

Permission to use, copy, distribute and modify this software for any purpose
with or without fee is hereby granted. We request, however, that all derived
work reference the NAS Parallel Benchmarks 3.0. This software is provided "as
is" without express or implied warranty.

Information on NPB 3.0, including the Technical Report NAS-02-008
"Implementation of the NAS Parallel Benchmarks in Java", original
specifications, source code, results and information on how to submit new
results, is available at:

   http://www.nas.nasa.gov/Software/NPB/'! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 16:36'!
master
	^ master! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 16:37'!
master: val
	master := val! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/17/2009 08:30'!
maxKey
	^ maxKey! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/17/2009 08:30'!
maxKey: val
	maxKey := val! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/17/2009 08:28'!
maxKeyLog2
	^ maxKeyLog2 ! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/17/2009 08:28'!
maxKeyLog2: val
	maxKeyLog2 := val! !

!NPBISBase methodsFor: 'synchronization' stamp: 'StefanMarr 10/5/2009 18:13'!
notify
	mtxNotify signal ! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 15:21'!
numBuckets 
	^ numBuckets! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 15:21'!
numBuckets: val
	numBuckets := val! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/17/2009 08:29'!
numBucketsLog2 
	^ numBucketsLog2 ! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/17/2009 08:29'!
numBucketsLog2: val
	numBucketsLog2 := val! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 15:21'!
numKeys
	^ numKeys! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 15:36'!
numKeys: val
	numKeys := val! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 16:02'!
numThreads
	^ numThreads! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 16:02'!
numThreads: val
	numThreads := val! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/23/2009 19:32'!
partialVerifyVals
	^partialVerifyVals ! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 16:37'!
rankThreads
	^rankThreads ! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 16:37'!
rankThreads: val
	rankThreads := val! !

!NPBISBase methodsFor: 'initialization' stamp: 'sm 6/1/2011 15:52'!
setupThreads
	"Initialize the threads/processes for the benchmark"
	
	| start end remainder offset rStart rEnd rRemainder rOffset rThread |
	
	start := 0. end := 0. remainder := totalKeys \\ numThreads. offset := 0.
	rStart := 0. rEnd := 0. rRemainder := maxKey \\ numThreads. rOffset := 0.
	
	rankThreads := Array new: numThreads.
	
	1 to: numThreads do: [ :i |
		| zeroIdx |
		zeroIdx := i - 1.
		start := (zeroIdx * ((totalKeys / numThreads) asInteger)) + offset.
		end :=  ((zeroIdx * ((totalKeys / numThreads) asInteger))) + ((totalKeys / numThreads) asInteger) - 1 + offset.
		(remainder > 0) ifTrue: [
			remainder := remainder - 1.
			offset := offset + 1.
			end := end + 1].
		
		rStart := (zeroIdx * ((maxKey / numThreads) asInteger)) + rOffset.
		rEnd := ((zeroIdx * ((maxKey / numThreads) asInteger))) + ((maxKey / numThreads) asInteger) -1 + rOffset.
		
		(rRemainder > 0) ifTrue: [
			rRemainder := rRemainder - 1.
			rOffset := rOffset + 1.
			rEnd := rEnd + 1].
		
		rThread := RankThread newFor: self with: i start: (start + 1) end: (end + 1) rStart: (rStart + 1) rEnd: (rEnd + 1).
		rankThreads at: i put: rThread.
		rThread rankThreads: rankThreads.
		rThread start.
	].! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 15:22'!
sizeOfBuffers
	^ sizeOfBuffers! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 15:22'!
sizeOfBuffers: val
	sizeOfBuffers := val! !

!NPBISBase methodsFor: 'accessors' stamp: 'SM 9/16/2009 22:50'!
testIndexArray
	^ testIndexArray! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 1/16/2011 20:47'!
testIndexArray: val
	testIndexArray := val! !

!NPBISBase methodsFor: 'accessors' stamp: 'SM 9/16/2009 22:51'!
testRankArray
	^ testRankArray! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 1/16/2011 20:46'!
testRankArray: val
	testRankArray := val! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/17/2009 08:30'!
totalKeys
	^ totalKeys! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 9/21/2009 15:38'!
totalKeys: val
	totalKeys := val! !

!NPBISBase methodsFor: 'accessors' stamp: 'SM 9/16/2009 22:54'!
totalKeysLog2
	^totalKeysLog2! !

!NPBISBase methodsFor: 'accessors' stamp: 'StefanMarr 1/16/2011 20:46'!
totalKeysLog2: val
	totalKeysLog2 := val! !

!NPBISBase methodsFor: 'synchronization' stamp: 'StefanMarr 10/5/2009 18:12'!
wait
	mtxNotify wait ! !

NPBISBase subclass: #RankThread
	instanceVariableNames: 'id localHist start end rStart rEnd state'
	classVariableNames: 'Iteration'
	poolDictionaries: ''
	category: 'NPB-IS'!

!RankThread class methodsFor: 'instance variables' stamp: ''!
iteration
	^ Iteration! !

!RankThread class methodsFor: 'instance variables' stamp: ''!
iteration: val
	Iteration := val! !

!RankThread class methodsFor: 'benchmarking' stamp: ''!
newFor: anIS with: id start: s1 end: e1 rStart: s2 rEnd: e2
	"create new instance"

	| o |

"RVMOperations print: ('new RankThread: ', id asString, ' ', s1 asString, ' ', e1 asString, ' ', s2 asString, ' ', e2 asString)."

	o := self new.
	
	o initializeValues: anIS.
	
	o master: anIS.
	o id: id.
	o start: s1.
	o end: e1.
	o rStart: s2.
	o rEnd: e2.
	o localHist: (Array new: o maxKey).
	o state: 0.
	
	^o
"setPriority(Thread.MAX_PRIORITY);
setDaemon(true);"! !

!RankThread methodsFor: 'accessing' stamp: ''!
end: val
	end := val! !

!RankThread methodsFor: 'accessing' stamp: ''!
id: val
	id := val! !

!RankThread methodsFor: 'as yet unclassified' stamp: ''!
initKeys
	| rng mySeed a x k |
	a := NPBIS amult.
	rng := NPBRandom new.
	mySeed := rng findSeedFor: (id - 1) with: numThreads of: (4 * totalKeys * 1) using: (NPBRandom seed) and: a.
	
	rng seed: mySeed.
	
	k := (maxKey / 4) asInteger.
    
	((start + numKeys) > (end + 1)) ifTrue: [self error: 'this here seems to be inconsistent'.].
	
	start to: end do: [:i |
		x := rng randlc: a.
		x := x + (rng randlc: a).
		x := x + (rng randlc: a).
		x := x + (rng randlc: a).
		keyArray at: i put: (x * k) asInteger.
		]! !

!RankThread methodsFor: 'initialization' stamp: ''!
initializeValues: anIS
	numThreads := anIS numThreads.
	maxKey := anIS maxKey.
	
	keyArray := anIS keyArray.
	testIndexArray := anIS testIndexArray.
	masterHist := anIS masterHist.
	partialVerifyVals := anIS partialVerifyVals.
	totalKeys := anIS totalKeys.
	numKeys := (anIS numKeys / anIS numThreads * 1) asInteger.
! !

!RankThread methodsFor: 'license' stamp: 'StefanMarr 4/15/2011 17:04'!
license
	^'This code is based on the Java version of NPB3.

Permission to use, copy, distribute and modify this software for any purpose
with or without fee is hereby granted. We request, however, that all derived
work reference the NAS Parallel Benchmarks 3.0. This software is provided "as
is" without express or implied warranty.

Information on NPB 3.0, including the Technical Report NAS-02-008
"Implementation of the NAS Parallel Benchmarks in Java", original
specifications, source code, results and information on how to submit new
results, is available at:

   http://www.nas.nasa.gov/Software/NPB/'! !

!RankThread methodsFor: 'accessing' stamp: ''!
localHist
	^localHist ! !

!RankThread methodsFor: 'accessing' stamp: ''!
localHist: val
	localHist := val! !

!RankThread methodsFor: 'accessing' stamp: ''!
rEnd
	^rEnd! !

!RankThread methodsFor: 'accessing' stamp: ''!
rEnd: val
	rEnd := val! !

!RankThread methodsFor: 'accessing' stamp: ''!
rStart
	^rStart! !

!RankThread methodsFor: 'accessing' stamp: ''!
rStart: val
	rStart := val! !

!RankThread methodsFor: 'running' stamp: ''!
run
	[true] whileTrue: [
		"RVMOperations print: 'RankThread waiting id=.', id asString."
		self wait.
		
		self critical: [
			"RVMOperations print: 'RankThread entered run critical Id=', id asString."
					
			(state == #quit)
				ifTrue: [
					master notify. 
					^self
				]
				ifFalse: [
					self perform: state.
					master notify.
				].
		]
	]! !

!RankThread methodsFor: 'accessing' stamp: ''!
start
	[self run] fork.! !

!RankThread methodsFor: 'accessing' stamp: ''!
start: val
	start := val! !

!RankThread methodsFor: 'running' stamp: 'sm 6/1/2011 15:14'!
startStep: aStep
	self critical: [
		self state: aStep.
		self notify.
	].! !

!RankThread methodsFor: 'accessing' stamp: ''!
state: val
	state := val! !

!RankThread methodsFor: 'as yet unclassified' stamp: ''!
step1
	"RVMTester print: 'do step 1 '."
	
	keyArray at: (Iteration + 1) put: Iteration.
	keyArray at: (1 + Iteration + (NPBISBase maxIterations)) put: (maxKey - Iteration).
	
	1 to: NPBISBase testArraySize do: [
		:i | partialVerifyVals at: i put: (keyArray at: (1 + (testIndexArray at: i)))].
	
	1 to: maxKey do: [
		:i | localHist at: i put: 0].
	
	start to: end do: [
		:i | localHist at: (1 + (keyArray at: i)) incrementBy: 1].
	
	1 to: (maxKey - 1) do: [
		:i | localHist at: (i + 1) incrementBy: (localHist at: i)]! !

!RankThread methodsFor: 'as yet unclassified' stamp: ''!
step2
	"RVMTester print: 'do step 2 '."
	rStart to: rEnd do: [
		:i | 1 to: numThreads do: [
			:j | masterHist at: i incrementBy: ((rankThreads at: j) localHist at: i)]]! !

Object subclass: #NPBRandom
	instanceVariableNames: 'tran amult KS r23 r46 t23 t46'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NPB'!

!NPBRandom class methodsFor: 'as yet unclassified' stamp: ''!
d2m46
	^ 0.5 raisedTo: 46! !

!NPBRandom class methodsFor: 'as yet unclassified' stamp: ''!
i246m1
	^ (2 raisedTo: 46) - 1! !

!NPBRandom class methodsFor: 'as yet unclassified' stamp: ''!
seed
	^314159265.0! !

!NPBRandom methodsFor: 'as yet unclassified' stamp: ''!
findSeedFor: aRank with: numOfThreads of: totalRanNumbers using: seed and: aGenMult
	| i t1 t2 an 
	 mq nq kk ik continueLoop |

	nq := (totalRanNumbers / numOfThreads) asInteger.
	mq := 0.

	[nq > 1] whileTrue: [
		mq := mq + 1.
		nq := nq / 2.
	].

	t1 := aGenMult.
	
	1 to: mq do: [ :ii |
		tran := t1.
		t2 := self randlc: t1.
		t1 := tran.
	].

	an := t1.
	
	kk := aRank.
	t1 := seed.
	t2 := an.

	i := 1.
	
	continueLoop := true.
	[(i <= 100) and: continueLoop] whileTrue: [
		ik := (kk / 2) asInteger.
		
		((2 * ik) ~= kk) ifTrue: [
			tran := t1.
			self randlc: t2.
			t1 := tran.].
		
		(ik == 0) ifTrue: [ continueLoop := false. ]
		ifFalse: [
			tran := t2.
			self randlc: t2.
			t2 := tran.
			kk := ik.
			i := i + 1.
			].
	].
   
	^ t1.! !

!NPBRandom methodsFor: 'initialization' stamp: ''!
initialize
	tran := NPBRandom seed.
	amult := NPBIS amult.
	KS := 0.
	r23 := 0.5 raisedTo: 23.
	r46 := r23 raisedTo: 2.
	t23 := 2 raisedTo: 23.
	t46 := t23 raisedTo: 2.! !

!NPBRandom methodsFor: 'license' stamp: 'StefanMarr 4/15/2011 17:04'!
license
	^'This code is based on the Java version of NPB3.

Permission to use, copy, distribute and modify this software for any purpose
with or without fee is hereby granted. We request, however, that all derived
work reference the NAS Parallel Benchmarks 3.0. This software is provided "as
is" without express or implied warranty.

Information on NPB 3.0, including the Technical Report NAS-02-008
"Implementation of the NAS Parallel Benchmarks in Java", original
specifications, source code, results and information on how to submit new
results, is available at:

   http://www.nas.nasa.gov/Software/NPB/'! !

!NPBRandom methodsFor: 'as yet unclassified' stamp: ''!
randlc: aDouble
	| t1 t2 t3 t4 a1 a2 x1 x2 z |
	" Break A into two parts such that A = 2**23 * A1 + A2. "

	t1 := r23 * aDouble.
	a1 := t1 asInteger.
	a2 := aDouble - (t23 * a1).
	
	" Break X into two parts such that X = 2^23 * X1 + X2, compute
	  Z = A1 * X2 + A2 * X1  (mod 2^23), and then
	  X = 2^23 * Z + A2 * X2  (mod 2^46). "

	t1 := r23 * tran.
	x1 := t1 asInteger.
	x2 := tran - (t23 * x1).
	t1 := (a1 * x2) + (a2 * x1).
	t2 := (r23 * t1) asInteger.
	z := t1 - (t23 * t2).
	t3 := (t23 * z) + (a2 * x2).
	t4 := (r46 * t3) asInteger.
	tran := t3 - (t46 * t4).
	^ r46 * tran! !

!NPBRandom methodsFor: 'accessing' stamp: ''!
seed
	^ tran! !

!NPBRandom methodsFor: 'private - accessing' stamp: ''!
seed: aSeed
	tran := aSeed! !

SMarkSuite subclass: #NPBSuite
	instanceVariableNames: 'bench'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NPB'!

!NPBSuite class methodsFor: 'transporter' stamp: 'StefanMarr 4/5/2012 21:48'!
transportersForFileOutMenu
	^ { (Smalltalk at: #Transporter ifAbsent: [^#()])
			forPackage: (PackageInfo named: 'NPB') }! !

!NPBSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 5/14/2011 01:00'!
benchIS
	"Run the IS benchmark"
	bench run.! !

!NPBSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 5/14/2011 10:43'!
setUpBenchIS
	"Prepare IS benchmark"
	| sizeClass threadCnt |
	
	sizeClass := runner problemSize.
	sizeClass ifNil:	[ sizeClass := NPBIS defaultProblemSize. ].
	
	threadCnt := runner processes.
	
	bench := NPBIS newOfSizeClass: sizeClass with: threadCnt serial: (threadCnt == 0).
	bench runner: runner.! !

TestCase subclass: #NPBTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NPB-Tests'!

!NPBTest methodsFor: 'running' stamp: 'StefanMarr 5/22/2011 10:23'!
setUp
	"Necessary for Squeak 4.2 compatibility, since method anotations are not supported in the base image."
	(self respondsTo:  #timeout:) ifTrue: [
		self timeout: 300.
	].! !

!NPBTest methodsFor: 'tests' stamp: ''!
testIS
	| verified runner suite |
	runner := SMarkRunner new.
	suite  := NPBSuite new.
	suite runner: runner.
	runner suite: suite.
	runner iterations: 1.
	runner processes: 4.
	runner problemSize: #S.
	
	self shouldnt: [	runner execute. 
				"TODO: update as soon as the SMark framework supports verification"
				verified := false. ]
		 raise: Error.
	self assert: verified.! !

!NPBTest methodsFor: 'tests' stamp: ''!
testKeyArrayContent
	| o keyArray |
	o := NPBIS newOfSizeClass: #S with: 0 serial: true.
	o initKeys: NPBIS amult.
	
	keyArray := o keyArray.
	
	self assert: (keyArray at: 65536) equals: 962.
	self assert:	(keyArray at: 65535) equals: 677.
	self assert: (keyArray at: 30001) equals: 557.
	self assert: (keyArray at: 1) equals: 1585.
	self assert: (keyArray at: 2) equals: 825.
	self assert: (keyArray at: 12) equals: 1140.
! !
